#ifndef _dialogo_
#define _dialogo_

/* Identificadores de los controles usados*/
enum {
	ID_NOMBRE=8020
};
#include "elemento.h"

#include "wx/wx.h"
#include "wx/spinctrl.h"

/* 	Clase dialogo
	
	Marca las directrices bsicas para los distintos cuadros de dialogo de
	configuracion de los elementos
*/

class Dialogo : public wxDialog
{
	/*	Esta es la constante de identificacin asignada por defecto a 
		controles que no se van a programar. El nombre est elegido al
		azar y el valor tambin */
	#define ID_STATIC	6989
	public:
		DECLARE_CLASS (Dialogo)
		Dialogo ();		
		Dialogo(	wxWindow* ventanaPadre, wxWindowID id,
					const wxString& titulo	= wxT ("Configurar"),
					const wxPoint&  posicion= wxDefaultPosition,
					const wxSize&   tam		= wxDefaultSize,
					long  			estilo	= wxCAPTION | wxRESIZE_BORDER | 
												wxSYSTEM_MENU );
		void Init ();

		bool Create ( wxWindow* ventanaPadre, wxWindowID,
				const wxString& titulo	= wxT ("Configurar"),
				const wxPoint&  posicion= wxDefaultPosition,
				const wxSize&   tam		= wxDefaultSize,
				long  			estilo	= wxCAPTION | wxRESIZE_BORDER | 
											wxSYSTEM_MENU );
		
	
		
			
		/*	Los controles de los distintos cuadros de dilogo se construyen
			de distinta forma de unos dilogos a otros */
		virtual void ConstruirControles()=0;
		
		
		/* 	Estos metodos transfieren informacion entre un elemento y las 
			propiedades establecidas en el cuadro de dilogo */
	
		/*	Lee el nombre y valores del elemento y los representa en
			el cuadro de dilogo */
		virtual void getPropiedades ( Elemento* elemento )=0;
		
		/*	Hace que un elemento adquiera las propiedades del dilogo */
		virtual void setPropiedades ( Elemento* elemento )=0;
		
	protected:
		/*	Este metodo proporciona un encabezamiento estandar con
			una descripcion, un TextControl para escribir el nombre del
			elemento y unos botones estandar aceptar y cancelar. El parametro
			que acepta es un contenedor cualquiera en el cual haya controles.
			El metodo lo cogera y lo pondra en el centro */
		void ConstruirCuadroDeDialogo (wxSizer* contenedor);
		/*	Los controles. */
		
		/* 	Los contenedores: Hay un contenedor principal que apila todos
			los demas controles en vertical de la siguiente forma
				1.- El texto descriptivo 
				2.- El contenedor central nos lo pasaran por medio
					del metodo ConstruirCuadroDeDialogo
				3.- El contenedor de los botones Aceptar Cancelar*/
		wxBoxSizer* contenedorPrincipal;
		
		
		wxStaticText* lblTextoDescriptivo ;
		
		wxBoxSizer*	contenedorNombre;
		wxStaticText* lblNombre; /*Contiene el texto "Introduzca los datos..."*/
		wxTextCtrl*	txtNombre; /*Control donde el usuario escribe el nombre*/
	
		wxBoxSizer*	contenedorBotones; /*Contiene los botones Aceptar/Cancelar*/
	
		/* Aqui se almacena el texto del nombre del elemento*/
		wxString textoNombre;
		
		
		wxButton*	botonAceptar;
		wxButton*	botonCancelar;
		
		/*	Este es el estilo que van a tener los botones. Por defecto
			no tienen ningun estilo especial, aunque esto se puede cambiar
			asignando otros valores en lugar de 0, como las constantes
			wxSUNKEN_BORDEN, etc... Consultar los estilos de ventana en
			la documentacion de wx */
		#define ESTILO	0
		
		/*	Cantidad de pixeles dibujados alrededor de los botones*/
		#define TAM_BORDE 5
		
		#define VOLTAJE_MINIMO 			0
		#define VOLTAJE_MAXIMO			50
		#define VOLTAJE_EN_REPOSO		0
		#define VOLTAJE_EN_ACTIVACION	9	
		#define TIMEOUT_MINIMO			0
		#define TIMEOUT_MAXIMO			90
		/*	Aqui hay funciones muy utilizadas en los cuadros de dilogo.
			Se factorizan aqui algunas */
			
		/*	Esta funcion obtiene una referencia a un wxSpinCtrl, lee
			su valor y lo mete dentro de una seal voltaje pasada
			desde fuera */
		void TransferirDeSpinAVoltaje (wxSpinCtrl* spin, Voltaje* voltaje);
		void TransferirDeVoltajeASpin (wxSpinCtrl* spin, Voltaje* voltaje);
		
		void TransferirDeChoiceAConexion (wxChoice* chc, Conexion* c);
		void TransferirDeConexionAChoice (wxChoice* chc, Conexion* c);
		/*	Estas funciones pasan el nombre del control hacia un elemento
			y viceversa */
		void TransferirDeNombreDeElementoAControl 	( Elemento* e );
		void TransferirDeControlANombreDeElemento 	( Elemento* e );
		
		void TransferirDeGiroAControl	(wxChoice* chc, Giro* giro);
		void TransferirDeControlAGiro	(wxChoice* chc, Giro* giro);
		
		void TransferirDeChoiceAMagnetismo (wxChoice* chc, Magnetismo* m);
		void TransferirDeMagnetismoAChoice (wxChoice* chc, Magnetismo* m);
		
		void TransferirDeSpinAFrecuencia (wxSpinCtrl* spin, Frecuencia* frecuencia);
		void TransferirDeFrecuenciaASpin (wxSpinCtrl* spin, Frecuencia* frecuencia);
		
		void TransferirDeSpinAColor (wxSpinCtrl* spinR, wxSpinCtrl* spinG,
							wxSpinCtrl* spinB, Luminosidad* color);
		void TransferirDeColorASpin (wxSpinCtrl* spinR, wxSpinCtrl* spinG,
							wxSpinCtrl* spinB, Luminosidad* color);
};

#endif
