/*	Fichero dialogolampara.h
	
	Contiene el interfaz para el cuadro de dialogo que permite configurar
	una lampara
	
	Ultima actualizacion: Martes 7-11-2006
*/


#ifndef _dialogolampara_h
#define _dialogolampara_h

#include "wx/wx.h"
#include "wx/statline.h"
#include "wx/spinctrl.h"
#include "wx/valgen.h"
/*	Este fichero es necesario para que el cuadro de dialogo pueda manejar
	los mismos tipos de seales que manejan los elementos */
#include "senal.h"

#include "dialogo.h"
#include "lampara.h"

/* Identificadores numricos asignados a los botones del cuadro de dialogo*/
enum {
	ID_DIALOGO_LAMPARA=8900,
	ID_B_ACTIVACION,
	ID_B_REPOSO ,
	ID_COLOR_ACTIVACION ,
	ID_COLOR_REPOSO ,
	ID_G_ACTIVACION ,
	ID_G_REPOSO ,
	ID_R_ACTIVACION ,
	ID_R_REPOSO ,
	ID_VOLTAJE_ACTIVACION ,
	ID_VOLTAJE_REPOSO 
};

class DialogoLampara : public Dialogo
{
	DECLARE_CLASS (DialogoLampara)
	DECLARE_EVENT_TABLE()
	public: 
		DialogoLampara ();
		DialogoLampara ( 	wxWindow* ventanaPadre, wxWindowID id=ID_DIALOGO_LAMPARA,
							const wxString& titulo= wxT ("Configurar lampara"),
							const wxPoint&  posicion=wxDefaultPosition,
							const wxSize&   tam=wxDefaultSize,
							long  			estilo=wxCAPTION | 
													wxRESIZE_BORDER |
													wxSYSTEM_MENU );

		
		void ConstruirControles();
		void Init ();

		/* Metodos getter y setter */
		
		/*	Lee el nombre y valores del elemento y los representa en
		el cuadro de dilogo */
		virtual void getPropiedades ( Elemento* elemento );
	
		/*	Hace que un elemento adquiera las propiedades del dilogo */
		virtual void setPropiedades ( Elemento* elemento );
	
	protected:
		/*	Gestin de eventos */
		
		void OnColorReposo		(wxCommandEvent& evento);
		void OnColorActivacion	(wxCommandEvent& evento);
		
		/*	Este metodo se usa para trabajar de igual forma con el color
			en reposo que con el color en activacion */
		void ManejarDialogoColor (wxSpinCtrl* r, wxSpinCtrl* g, wxSpinCtrl* b);
	private:
		
		/*	Estos nombres de variable no son del todo representativos,
			ya que el usuario puede invertir los valores y escribir que
			por ejemplo 0V es el voltaje que el pulsador emite cuando se
			pulsa. Se usan estos nombres porque por defecto se usaran
			0V para el reposo y "false" para el estado no-conectado*/
		char	nombreElemento[MAX_LONG_NOMBRE_ELEMENTO];
		
		wxBoxSizer *contenedorCentral;
		wxBoxSizer *contenedorReposo;
		wxBoxSizer *contenedorActivacion;
		
		wxStaticText *lblVApagado;
	    wxSpinCtrl *spnVReposo;
	    wxStaticText *lblColorInicial;
	    wxStaticText *lblRReposo;
	    wxSpinCtrl *spnRReposo;
	    wxStaticText *lblGReposo;
	    wxSpinCtrl *spnGReposo;
	    wxStaticText *lblBReposo;
	    wxSpinCtrl *spnBReposo;
	    wxButton *btnColorReposo;
	    wxStaticText *lblVActivacion;
	    wxSpinCtrl *spnVActivacion;
	    wxStaticText *lblCActivacion;
	    wxStaticText *lblRActivacion;
	    wxSpinCtrl *spnRActivacion;
	    wxStaticText *lblGActivacion;
	    wxSpinCtrl *spnGActivacion;
	    wxStaticText *lblBActivacion;
	    wxSpinCtrl *spnBActivacion;
	    wxButton *btnColorActivacion;
  		
		
};

#endif


