/*	Fichero elementovisual.h
	Contiene la definicin basica de la clase que permite que los elementos
	puedan representarse en pantalla encapsulando los elementos mediante
	un patrn decorator. As, un PulsadorVisual ser una clase que
	almacenar un Pulsador dentro. Es necesario que herede de Elemento pero
	tambin de alguna parte de la jerarqua de WxWidgets */
	
#ifndef _elementovisual_h
#define _elementovisual_h

#include "wx/wx.h"
#include "wx/dcbuffer.h" /* Para poder usar un buffered paint */
#include "wx/dcclient.h"
#include "elemento.h"
#include "../dialogos/dialogo.h"
#include <xercesc/util/XMLString.hpp>

#include <vector>
/*	Aqui se definen parametros estandar para los iconos de los
	elementos visuales */
	
/*	Los iconos en general son pixmaps (es el tipo que incluyen por
	defecto las compilaciones de WxWidgets) */
#define TIPO_ICONOS		wxBITMAP_TYPE_XPM	

/*	Tamao en pixeles */
#define ANCHURA_ICONOS	48
#define ALTURA_ICONOS	48

/*	La profundidad de color estndar en bits. -1 significa "el que
	tenga la pantalla en ese momento */
#define PROFUNDIDAD		-1

/*	Numero total de elementos que pueden esperar su entrada
	a partir de la entrada de este elemento */
#define	MAX_SUSCRIPTORES	30


class ElementoVisual : public wxObject, public Elemento
{
	public:
		ElementoVisual();
		~ElementoVisual();
		
		/*	Si se ha usado un constructor por defecto, se puede hacer
			que un elemento visual dado "recubra" a este nuevo elemento e*/
		virtual void setElementoEnvuelto (Elemento* e);
		Elemento* 	 getElementoEnvuelto();
		/*	Estos metodos se heredan de la clase Elemento, pero
			en realidad solo redirigen peticiones al elemento envuelto.
			Consultar elementovisualdecorador.cpp */
		virtual int anadirSuscriptor   (Elemento* e,int pos_entrada=POS_ENTRADA, 
							int pos_salida=POS_SALIDA);
							
		/*	Se proporciona un manejador de entradas por defecto que
			muestra en pantalla la accion que va a tener lugar.*/
		virtual void manejarEntrada (Senal* entrada, int posicion);
		/*	Este metodo lo que en realidad har ser llamar a setEntrada
			del elemento encapsulado */
		virtual int setEntrada (Senal* entrada, int posicion=POS_ENTRADA);
		
		/* Nos dicen si una entrada o salida es un voltaje o giro de motor...*/
		virtual int 	getTipoEntrada(int posicion=POS_ENTRADA);
		virtual int 	getTipoSalida (int posicion=POS_SALIDA);
		
		int 			getTotalEntradas();
		int 			getTotalSalidas ();
		int 			getTotalSuscripciones();
		Suscripcion**	getSuscripciones();
		void			setX(int x);
		void			setY(int y);
		
		/*	Permite conocer el nombre de las entradas y salidas, as
			como las posiciones que ocupan */
		virtual map<string,int> getNombreEntradas	();
		virtual map<string,int> getNombreSalidas	();
		/*	Este mtodo permite construir un elemento a partir
			de un trocito de XML */
		virtual void construir (DOMElement*);
		/*****************************/
		/*	Fin de metodos heredados */
		/*****************************/
		wxBitmap 			GetBitmap() { return bitmapEnPantalla;}
		virtual void 		DibujarElemento (wxDC& dc);
		/* Cuidado, si se usa bufferedpaintdc puede haber problemas si
			hay una scrolledwindow */
		void 				DibujarRelaciones (wxDC& dc);
		
		/*	Estas funciones modifican el bitmap para que
			se pueda ver de forma visual que un elemento
			ha sido seleccionado */
		void				Seleccionar();
		void				Deseleccionar();
		/*	Permite configurar las propiedades del elemento pero se
			aprovecha del metodo virtual FabricarDialogo (un Factory Method)
			que es redefinido en cada elemento */
		virtual void		Configurar (Elemento* elemento);
		virtual void		Configurar();
		virtual Dialogo* 	FabricarDialogo() = 0;
		
		/*	En elementos con muchas salidas o entradas
			puede que la conexin no se haga a las posiciones
			de entrada o salida por defecto */
		virtual int			ElegirSalidaDeEmisor();		
		virtual int			ElegirEntradaDeReceptor();
		
		/*	Estos mtodos hacen un poco ms comodo el manejo
			de las posiciones desde WX*/
		wxPoint				GetXY();
		void				SetXY(wxPoint punto);
		void				SetXY(int x, int y); 
		
		void				SetVisible (bool visible);
		bool 				EsVisible ();
		/*	Devuelve un rectangulo con las coordenadas de la zona
			visual que ocupa este elemento */
		wxRect				GetRectangulo();
		/*	Cambia el nombre del elemento */
		virtual void		SetNombre (const char *nombre);
		virtual void		SetNombre (wxString nombre);
		virtual wxString	GetNombre ();
		/*	Aade al nombre del elemento un sufijo con el numero
			del elemento que le corresponde */
		virtual void		SetSufijo (int numelemento);
		
		/*	Establece cual va a ser el sistema padre */
		void 				SetPadre (Elemento* padre);
		Elemento*			GetPadre ();
		
		/*	Cuando otro elemento se convierte en suscriptor de este elemento
		esta conectando una cierta entrada suya a una cierta salida nuestra.
		Tambien se usan parametros por defecto*/
		int Conectar   (Elemento* e,int pos_entrada=POS_ENTRADA, 
								int pos_salida=POS_SALIDA);
		int Desconectar(Elemento* e,int pos_entrada=POS_ENTRADA, 
								int pos_salida=POS_SALIDA);
		
		/*	Cuando alguien hace click encima del elemento, conmuta
			su entrada. Es decir, si estaba en reposo pasa a estar
			en activacion y al revs */		
		virtual void 	ConmutarEntrada();
		wxString		getInformacion();
		virtual NodoXML getXML ( DOMImplementation* impl );
		virtual void 	getXMLSuscriptores (DOMDocument *documento_xml);
		/* No siempre se usa. Algunos elementos como el temporizador
			pueden usarlo para avisar a la pagina padre de que
			se debe hacer un repintado o cambio de foco */
		void 			SetVentanaPadre (wxWindow* padre);
	protected:
		/* La mayor parte de elementos tienen solo dos estados. Aqui
			se guardaran las seales que indican cuando un elemento
			debe pasar a estado de reposo o de activacion */
		Senal* senalReposo;
		Senal* senalActivacion;
		
		wxWindow* paginaPadre;
		Elemento* elementoEnvuelto;
		enum estado
		{
			INACTIVO, ACTIVO, SELECCIONADO 
		};
		estado estado_actual;
		/*	Esto permite que los elementos tengan un nombre unico asignado
			por el sistema. Al final no lo us pero est disponible para
			las clases hija */
		void asignarNombreUnico (const char* nombreDeElemento, int numero);
		
		/*	Esto dibuja una lnea que conecta dos elemento unidos por
			una suscripcion. Se hace as por si se desea cambiar el 
			dibujo concreto de la lnea */
		void				DibujarSuscripcion (Elemento* e,wxDC& contexto);
		
		/*	Una referencia al objeto padre que lo contiene. Es necesario
			para poder componer elementos recursivamente */
		Elemento*	contenedorPadre;
		/*	Este es el icono del elemento que se debe mostrar en pantalla*/
		wxBitmap	bitmapEnPantalla;
		wxBitmap	bitmapElementoInactivo;
		wxBitmap	bitmapElementoActivo;
		
		/*	Permite configurar las propiedades del elemento */
		Dialogo*	dialogoDeConfiguracion;
		
		/*	El elemento es visible o no */
		bool		es_visible;
		wxString 	nombreDelElementoVisual;
		wxFont		fuente; /*Tipo de letra para escribir los nombres */
};

#endif
