#include "contador.h"


Contador::Contador ()
{
	voltajeEntradaReposo		=	NULL;
	voltajeEntradaActivacion	=	NULL;
	voltajeSalidaReposo			=	NULL;
	voltajeSalidaActivacion		=	NULL;
	
	this->autoReset=false;
	
	this->maxPulsos=-1;

	cuentaPulsos=0;
	
	entradas[POS_ENTRADA]=NULL;
	salidas [POS_SALIDA] =NULL;
}
Contador::~Contador()
{
	printf ("Destructor de %s\n", nombreElemento);
}
Contador::Contador(unsigned int maxPulsos,Voltaje *v_e_reposo,
			Voltaje* v_e_activacion, Voltaje* v_s_reposo, 
			Voltaje* v_s_activacion, bool autoReset)
{
	voltajeEntradaReposo		=	v_e_reposo;
	voltajeEntradaActivacion	=	v_e_activacion;
	voltajeSalidaReposo			=	v_s_reposo;
	voltajeSalidaActivacion		=	v_s_activacion;
	
	this->autoReset=autoReset;
	
	this->maxPulsos=maxPulsos;

	cuentaPulsos=0;
	
	entradas[POS_ENTRADA]=voltajeEntradaReposo;
	salidas [POS_SALIDA] =voltajeSalidaReposo;
}

int Contador::setEntrada (Senal* s, int posicion)
{
	bool senal_es_nula = (s==NULL);
	assert (!senal_es_nula);
	assert (posicion>=0);
	assert (posicion<= totalEntradas);
	assert (maxPulsos>0);
	Voltaje* nuevaEntrada=(Voltaje*) s;
	/*	Si estan introduciendo una activacion en la posicion de reset se resetea
		la cuenta */
	if (posicion==POS_RESET)
	{
		if (nuevaEntrada->getValor()==voltajeEntradaActivacion->getValor())
		{
			cuentaPulsos=0;
			entradas[posicion]=nuevaEntrada;
			return ENTRADA_CAMBIADA;
		}
	}
	
	/*	Si por el contrario introducen algo en la entrada se examina
		la situacion*/
	if (posicion==POS_ENTRADA)
	{
		Voltaje *entradaActual=(Voltaje*) entradas[posicion];
		/*	Si lo que habia en la entrada era un "reposo" y se introduce
			una "activacion" actualizamos la cuenta de pulsos*/
		if ( (entradaActual->getValor()==voltajeEntradaReposo->getValor() ) && 
			(nuevaEntrada->getValor()==voltajeEntradaActivacion->getValor() ) )
		{
			entradas[posicion]=nuevaEntrada;
			cuentaPulsos++;
#ifdef DEBUG_CONTADOR
			printf ("Contador actualizado\n");
#endif		
			/*	Y si se ha llegado a la cuenta total se pone en la salida
				el valor correspondiente*/
			if (cuentaPulsos==maxPulsos)
			{
#ifdef DEBUG_CONTADOR
				printf ("Contador disparado!\n");
#endif
				salidas[POS_SALIDA]=voltajeSalidaActivacion;
				
				if (autoReset)
				{
					cuentaPulsos=0;
				}
			//	assert (false);
				return (Elemento::setEntrada(s, posicion) );
			}
		}
	}
	return (Elemento::setEntrada(s, posicion) );
}

void Contador::setMaxPulsos(int numPulsos)
{
	if (numPulsos<=0) {
		maxPulsos=1;
	}
	else {
		maxPulsos=numPulsos;
	}
}
int Contador::getNumPulsos()
{
	return cuentaPulsos;
}

int Contador::getMaxPulsos()
{
	return maxPulsos;
}

void Contador::reset()
{
	
}

Voltaje* Contador::getVoltajeEntradaReposo()
{
	return voltajeEntradaReposo;
}

Voltaje* Contador::getVoltajeEntradaActivacion()
{
	return voltajeEntradaActivacion;
}

Voltaje* Contador::getVoltajeSalidaReposo()
{
	return voltajeSalidaReposo;
}

Voltaje* Contador::getVoltajeSalidaActivacion()
{
	return voltajeSalidaActivacion;
}

NodoXML Contador::getXML ( DOMImplementation *impl )
{
	DOMDocument* doc = impl->createDocument();
	/*	Obtenemos un documento base que incluye el elemento, su nombreelemento
		y sus atributos de posicion x e y */
	doc = getDocumentoXMLBase ( doc, "contador", nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */
		
	/*	Cuidado: El numero de pulsos no es un voltaje, pero fabricamos
		un voltaje falso para poder volcar este numero a XML */
	anadirElementoXML (doc, "cuentamaxima",			new Voltaje(maxPulsos)		);
	anadirElementoXML (doc, "entradareposo",		voltajeEntradaActivacion 	);
	anadirElementoXML (doc, "entradaactivacion",	voltajeEntradaReposo	 	);
	anadirElementoXML (doc, "salidareposo",			voltajeSalidaReposo		 	);
	anadirElementoXML (doc, "salidaactivacion",		voltajeSalidaActivacion		);
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	fragmento->appendChild ( doc->getDocumentElement() );
	return fragmento;
}


void Contador::construir (DOMElement* nodo)
{
	obtenerDatosComunes (nodo);
	
	// Recurrimos a una funcion auxiliar que extrae el numero de pulsos
	leerMaxPulsos ( nodo );
	/* Obtenemos los valores de los distintos nodos */
	voltajeEntradaReposo	= (Voltaje*) getSenal 
					( VOLTAJE, "entradareposo", 		nodo );
	voltajeEntradaActivacion= (Voltaje*) getSenal 
					( VOLTAJE, "entradaactivacion", 	nodo );
	voltajeSalidaReposo		= (Voltaje*) getSenal 
					( VOLTAJE, "salidareposo", 		nodo );
	voltajeSalidaActivacion	= (Voltaje*) getSenal 
					( VOLTAJE, "salidaactivacion", 	nodo );
	
	entradas[POS_ENTRADA]	=voltajeEntradaReposo;
	salidas[POS_SALIDA]		=voltajeSalidaReposo;
}


unsigned int Contador::leerMaxPulsos ( DOMElement* elemento)
{
	// Este es el nodo XML que contiene el numero de pulsos a contar
	
	// Recuperamos todos los elementos con esta etiqueta...
	XMLCh* etiqueta = XMLString::transcode ( "cuentamaxima" )	;
	DOMNodeList* lista = elemento->getElementsByTagName ( etiqueta );
	
	//... y solo deberia haber uno en la posicion 0
	DOMNode *nodoNombre= lista->item(0);
		
	//Entramos en su contenido...
	nodoNombre=nodoNombre->getFirstChild();
	const XMLCh* contenidoEtiqueta=nodoNombre->getNodeValue();
	
	//... y lo parseamos en busca de un entero
	maxPulsos = XMLString::parseInt ( contenidoEtiqueta );
	return maxPulsos;
}
