#include "fotosensor.h"
#include "constantes.h"
#include <iostream>

using namespace std;
FotoSensor::FotoSensor ()
{
	luminosidadReposo		=	NULL;
	luminosidadActivacion	=	NULL;
	voltajeSalidaReposo		=	NULL;
	voltajeSalidaActivacion	=	NULL;
	
	entradas[POS_ENTRADA]=NULL;
	salidas [POS_SALIDA] =NULL;
}
FotoSensor::FotoSensor(Luminosidad* lReposo, Luminosidad* lActivacion,
			Voltaje* vReposo, Voltaje* vActivacion)
{
	luminosidadReposo		=	lReposo;
	luminosidadActivacion	=	lActivacion;
	voltajeSalidaReposo		=	vReposo;
	voltajeSalidaActivacion	=	vActivacion;
	
	entradas[POS_ENTRADA]=lReposo;
	salidas [POS_SALIDA] =vReposo;
}

Luminosidad* FotoSensor::getLuminosidadEntradaReposo()
{
	return luminosidadReposo;
}

Luminosidad* FotoSensor::getLuminosidadEntradaActivacion()
{
	return luminosidadActivacion;
}

Voltaje* FotoSensor::getVoltajeSalidaReposo()
{
	return voltajeSalidaReposo;
}

Voltaje* FotoSensor::getVoltajeSalidaActivacion()
{
	return voltajeSalidaActivacion;
}

int	FotoSensor::setEntrada (Senal* v, int posicion)
{
	Luminosidad *lEntrada=(Luminosidad*) v;
	const char* entrada 	= lEntrada->getValor();
	const char *activacion 	= luminosidadActivacion->getValor();
	/*	Solo se comparan los primeros 20 caracteres */
	int codComparacion=strncmp (entrada, activacion, 20);
	bool esLuminosidadDeActivacion=(codComparacion==0);
	
	if (esLuminosidadDeActivacion)
	{
		salidas[POS_SALIDA]=voltajeSalidaActivacion;
		
	}
	else {
		salidas[POS_SALIDA]=voltajeSalidaReposo;
	}
	return (Elemento::setEntrada(v, posicion));
}

NodoXML FotoSensor::getXML ( DOMImplementation *impl )
{
	//etqElemento se hereda por parte de XMLSerializable
	//etqElemento = XMLString::transcode ( "fotosensor" );
	
	DOMDocument* doc = impl->createDocument ();
	cout << "Creando documento base XML para electroimans\n";
	doc = getDocumentoXMLBase ( doc, "fotosensor", nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */
	cout << "Documento base fotosensor creado\n";
	anadirElementoXML (doc, "entradareposo", 		
								getLuminosidadEntradaReposo() 			);
	anadirElementoXML (doc, "salidareposo", 		
								getVoltajeSalidaReposo()				);
	cout << "XML fotosensor:Reposo creados\n";
	anadirElementoXML (doc, "entradaactivacion",	
								getLuminosidadEntradaActivacion() 		);
	anadirElementoXML (doc, "salidaactivacion",		
								getVoltajeSalidaActivacion() 			);
	cout << "XML fotosensor:Activacion creada\n";
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	fragmento->appendChild ( doc->getDocumentElement() );
	return fragmento;
}


void FotoSensor::construir (DOMElement* nodo)
{
	obtenerDatosComunes (nodo);
	
	/* Obtenemos los valores de los distintos nodos */
	voltajeSalidaReposo	= (Voltaje*) getSenal 
					( VOLTAJE, "salidareposo", 		nodo );
	voltajeSalidaActivacion	= (Voltaje*) getSenal 
					( VOLTAJE, "salidaactivacion",	nodo );
	luminosidadReposo		= (Luminosidad*) getSenal
					( LUMINOSIDAD, "entradareposo", nodo );
	luminosidadActivacion	= (Luminosidad*) getSenal
					( LUMINOSIDAD, "entradaactivacion", nodo );
	
	entradas[POS_ENTRADA]	=luminosidadReposo;
	salidas[POS_SALIDA]		=voltajeSalidaReposo;
}
