#include "constantes.h"
#include <iostream>

#ifndef _senal_
#define _senal_



using std::string;

#define NADA 		0	
#define VOLTAJE 	1
#define LUMINOSIDAD	2
#define GIRO 		3
#define MAGNETISMO	4
#define CONEXION	5
#define FRECUENCIA	6

/*	Estos son los posibles estados de giro del motor*/
#define	GIRO_DETENIDO 	100
#define GIRO_CW			101
#define GIRO_CCW		102
/*	Se fabrica una jerarqua de seales para distinguir las distintas
	seales de entrada y salida que pueden tener los distintos elementos*/

class Senal {
	public:
			virtual int getTipo() 				{return NADA; }
			virtual const char* toString() 		{return "XX" ; }
			
			/* Este metodo permite comparar dos seales*/
			virtual bool esIgualA (Senal* s)	{ return false; }
};

/*	Un voltaje es un tipo por derecho propio y su valor no puede 
	ser modificado. Si se es necesario se deber crear otro voltaje*/
class Voltaje : public Senal {
	private:
		float valor;
	public:
		/*  Por comodidad,se suministran funciones que permiten construir
			tipos desde valores y desde cadenas */
		Voltaje (float v) {
			this->valor=v;
		}
		Voltaje ( char* valor ) {
			this->valor= (float) atof( valor );
		}
		float getValor() 	{ return valor ; }
		void  setValor(float v)	
		{
			valor=v;
		}
		
		virtual int getTipo () 		{ return VOLTAJE; }
		const char* toString() {
			char* c=new char[25];
			sprintf (c, "%.0f", valor);
			return c;
		}
};

/*	La luminosidad es una seal producida por la lmpara o la
	captada por un fotosensor. Se puede distinguir luminosidad
	de distintos colores*/
class Luminosidad: public Senal {
	private:
		char* color;
	public:
		Luminosidad (char* c) {
			color=new char[MAX_LONG_NOMBRE_ELEMENTO];
			strcpy (color,c);
		}
		const char* getValor() 		{ return color ; 		}
		void setValor(char* nuevocolor)
		{ 
			strncpy (color, nuevocolor, MAX_LONG_NOMBRE_ELEMENTO );
		}
		virtual int getTipo	()		{ return LUMINOSIDAD ; 	}
		const char* toString() 		{ return color;			}
		virtual bool esIgualA (Senal* s)
		{
			const char* valor1 = toString();
			const char* valor2 = s->toString();
			
			int codigo=strcmp (valor1, valor2);
			
			if (codigo==0) return true;
			
			return false;
		}
};

/*	Esta clase representa el giro de un motor. Cabe destacar que si no 
	se suministra un giro vlido, automticamente se pone el movimiento
	a "stop" */
class Giro : public Senal {
	private:
		char giro[MAX_LONG_PALABRA_GIRO];
	public:
		Giro (char* c)
		{
			strcpy (giro, c);
		}
		
		int getValor ()  { 
			int stop	;
			int cw 		;
			int ccw		;
			stop = strcmp (giro, "stop");
			if ( stop == 0) return GIRO_DETENIDO;
			
			cw = strcmp (giro, "cw");
			if ( cw == 0) return GIRO_CW;
			
			ccw = strcmp (giro, "ccw");
			if ( ccw== 0) return GIRO_CCW;
			
			/* En cualquier otro caso, asumimos que el motor esta parado */
			return GIRO_DETENIDO;
		}
		
		void setValor (char *c) {strcpy (giro,c);}
					
		virtual int getTipo()		{ return GIRO ; }
		
		const char* toString() {
			return giro;
		}
};



class Magnetismo : public Senal {
	private:
		bool valor;
	public:
		Magnetismo (bool v) { this->valor=v; }
		bool getValor () 	{ return valor; }
		void setValor (bool v) {this->valor=v;}
		virtual int getTipo ()		{ return MAGNETISMO; }
		const char* toString() {
			char* c=new char[10];
			if (valor==true) strcpy (c,"on");
			if (valor==false)strcpy (c,"off");
			return c;
		}
};

/*	Esta seal se usa en los pulsadores y en los rels indicando si
	el pulsador o un rel se ha conectado*/
class Conexion : public Senal {
	private:
		bool valor;
	public:
		Conexion (bool v) 			{ this->valor=v; }
		bool getValor ()			{ return valor; }
		void setValor (bool v) 		{this->valor=v;}
		virtual int getTipo ()		{ return CONEXION; }
		
		const char* toString() 		{
			char* c=new char[15];
			if (valor==true) strcpy (c,"conectado");
			if (valor==false)strcpy (c,"no-conectado");
			return c;
		}
};	
class Frecuencia : public Senal {
	private:
		unsigned int hercios;
	public:
		Frecuencia (int v) {
			this->hercios=v;
		}
		Frecuencia ( char* valor ) {
			this->hercios = (int) atoi( valor );
		}		
		
		unsigned int getValor() 	{ return hercios ; }
		virtual  int getTipo () 	{ return FRECUENCIA; }
		
		void  setValor(int v)	
		{
			hercios=v;
		}
		
		
		const char* toString() {
			char* c=new char[10];
			sprintf (c, "%i", hercios);
			return c;
		}
};


#endif
