#include "temporizador.h"


Temporizador::Temporizador()
{
	voltajeEntradaInicial	=	NULL;
	voltajeSalidaInicial	=	NULL;
	voltajeActivacion		=	NULL;
	voltajeSalidaEnTimeout	=	NULL;
	tiempoMax=0;
	entradas[POS_ENTRADA]	= voltajeEntradaInicial;
	salidas [POS_SALIDA]	= voltajeSalidaInicial;
	
	/*Si el tiempo empezara en 0, el temporizador tendria que 
	esperar siempre un pulso mas */						
	tiempoActual=1; 
	estoyActivado = false;
	//etqElemento se hereda por parte de XMLSerializable
}

Temporizador::Temporizador(Voltaje* vEntradaInicial, Voltaje* vSalidaInicial,
					Voltaje* vActivacion, Voltaje* vSalidaTimeout, 
					unsigned long tiempo)
{
	voltajeEntradaInicial	=	vEntradaInicial;
	voltajeSalidaInicial	=	vSalidaInicial;
	voltajeActivacion		=	vActivacion;
	voltajeSalidaEnTimeout	=	vSalidaTimeout;
	//etqElemento se hereda por parte de XMLSerializable
	tiempoMax=tiempo;
	tiempoActual=1;
	entradas[POS_ENTRADA]	= voltajeEntradaInicial;
	salidas [POS_SALIDA]	= voltajeSalidaInicial;
}

void Temporizador::reset()
{
	tiempoActual=1;
	entradas[POS_ENTRADA]	= voltajeEntradaInicial;
	salidas [POS_SALIDA]	= voltajeSalidaInicial;
}

/*	Se actualiza el tiempo actual y si se ha alcanzado el tiempo lmite o
	timeout, se notifica a los suscriptores y se reinicializa el temporizador*/
void Temporizador::actualizar()
{
//	printf ("Temporizador actualizado que iba por la cuenta %i",tiempoActual);
	/*	El temporizador solo trabaja si alguien lo ha activado con
		el voltaje necesario (ver setEntrada) */
	if (!estoyActivado) return ;
	
	if (tiempoActual==tiempoMax)
	{
//		printf ("Temporizador propagando!");
		salidas[POS_SALIDA]=voltajeSalidaEnTimeout;
		notificar();
		tiempoActual++; /*Con esto no volveremos a entrar aqui la proxima vez */
//		tiempoActual=0;
	}
	if (tiempoActual<tiempoMax) {
		tiempoActual++;
//		printf (" y ahora va por %i (max %i)\n", tiempoActual, tiempoMax);
	}
	
	/*	El temporizador no sigue notificando ni actualizando cosas
		hasta que alguien lo reinicie o si hay un autoreset quiza
		se deba reiniciar */
}

void Temporizador::construir (DOMElement* nodo)
{
	obtenerDatosComunes (nodo);
	
	voltajeEntradaInicial 	= (Voltaje*) getSenal (VOLTAJE, "voltajeinicial", nodo);
	voltajeSalidaInicial	= (Voltaje*) getSenal (VOLTAJE, "voltajesalidainicial", nodo);
	voltajeActivacion		= (Voltaje*) getSenal (VOLTAJE, "voltajeactivacion", nodo);
	voltajeSalidaEnTimeout	= (Voltaje*) getSenal (VOLTAJE, "voltajesalidaentemporizacion", nodo);
	/* Hack: No hay ningn metodo especial para leer un numero
		"pelado" como es el tiempo, pero lo leemos en una
		variable temporal...*/
	Voltaje* tiempo= (Voltaje*) getSenal (VOLTAJE, "tiempo", nodo);
	/*...y lo pasamos a la variable tiempoMax que es la que hay que rellenar*/
	tiempoMax= (int) tiempo->getValor();
	
	entradas[POS_ENTRADA]	= voltajeEntradaInicial;
	salidas[POS_SALIDA]		= voltajeSalidaInicial;
}
DOMElement*	Temporizador::getNodo()
{
	return Temporizador::getNodo("temporizador");
}
DOMElement* Temporizador::getNodo(char *nombreElemento)
{
	/* TODO: implementar esto */
	return NULL;
}

NodoXML Temporizador::getXML ( DOMImplementation* impl )
{
	/*	Hack peligroso si se modifica: El tiempo se "camufla" dentro de una seal
		de tipo voltaje, para as poder reaprovechar el metodo anadirElementoXML.
		El peligro radica en que el tiempo es un unsigned int (muy largo) y 
		podra ocurrir que hubiera errores con ciertas conversiones, aunque
		hasta ahora no me ha ocurrido */
	Voltaje* tiempo=new Voltaje(0.0);
	tiempo->setValor(tiempoMax);
	
	DOMDocument* doc= impl->createDocument ();	
	doc = getDocumentoXMLBase ( doc, "temporizador",nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */
	
	anadirElementoXML (doc, "voltajeinicial", 		getVoltajeEntradaInicial() 	);
		
	anadirElementoXML (doc, "voltajeactivacion",	getVoltajeActivacion() 		);
	
	anadirElementoXML (doc, "tiempo",				tiempo				 		);
	
	anadirElementoXML (doc, "voltajesalidainicial",	getVoltajeActivacion() 		);

	anadirElementoXML (doc, "voltajesalidaentemporizacion",
		 											getVoltajeSalidaEnTimeout()	);
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	DOMElement* elementoRaiz 	= doc->getDocumentElement();
	fragmento->appendChild ( elementoRaiz );
	return fragmento;
}
Voltaje*	 	Temporizador::getVoltajeEntradaInicial	()
{
	return voltajeEntradaInicial;
}
Voltaje*		Temporizador::getVoltajeSalidaInicial	()
{
	return voltajeSalidaInicial;
}
Voltaje*	 	Temporizador::getVoltajeActivacion		()
{
	return voltajeActivacion;
}
Voltaje*	 	Temporizador::getVoltajeSalidaEnTimeout	()
{
	return voltajeSalidaEnTimeout;
}
bool			Temporizador::esVoltajeDeActivacion		(Voltaje* voltajePasado)
{
	assert (voltajePasado!=NULL);
	assert (voltajeActivacion!=NULL);
	
	const char* vActivacion = voltajeActivacion->toString();
	const char* vPasado		= voltajePasado->toString();
	
	int codigo = strcmp (vActivacion, vPasado);
	
	if (codigo==0) return true;
	
	return false;
}
unsigned long int	Temporizador::getTiempoMax()
{
	return tiempoMax;
}

unsigned long int	Temporizador::getTiempoActual()
{
	return tiempoActual;
}

int Temporizador::setEntrada(Senal* entrada, int posicion)
{
	assert (entrada!=NULL);
	assert (posicion>=0);
	Voltaje* voltEntrada = (Voltaje*) entrada;
	float vEntrada = voltEntrada->getValor();
	
	float vActivacion 	= getVoltajeActivacion()->getValor();
	float vReposo		= getVoltajeEntradaInicial()->getValor();
	
	/*	Si el voltaje de entrada es el necesario para 
		iniciar la temporizacin... */
		
	/* Nota. El temporizador est continuamente emitiendo pulsos
		altos con 
	if (vEntrada==vActivacion)*/
	/*Con esta linea solo se activa una vez */
	if ((!estoyActivado) && (vEntrada==vActivacion))
	{
		/*	Se copian las entradas y se inicia la temporizacin
			si no se haba iniciado ya */
			entradas[POS_ENTRADA]	=	voltajeActivacion;
			estoyActivado 			= 	true;
			tiempoActual			=	1;
	}
	
	/*	En cambio si el voltaje es el de reposo... */
	
	if (vEntrada==vReposo)
	{
		/*	Se propaga en la salida el voltaje de reposo */
		entradas[POS_ENTRADA]	=	voltajeEntradaInicial;
		salidas[POS_SALIDA]		=	voltajeSalidaInicial;
		estoyActivado 			=	false;
	}
	
	/* TODO*/
	return 0;
}

void 	Temporizador::setVoltajeEntradaInicial	(Voltaje*	 voltaje)
{
	voltajeEntradaInicial=voltaje;
}
void	Temporizador::setVoltajeSalidaInicial	(Voltaje*	 voltaje)
{
	 voltajeSalidaInicial=voltaje;
}
void 	Temporizador::setVoltajeActivacion		(Voltaje*	 voltaje)
{
	 voltajeActivacion=voltaje;
}
void 	Temporizador::setVoltajeSalidaEnTimeout	(Voltaje*	 voltaje)
{
	 voltajeSalidaEnTimeout=voltaje;
}
void 	Temporizador::setTiempoMax	(unsigned long t)
{
	 tiempoMax=t;
}
