#include "unit++.h"
#include "biestable.h"
#include "pulsador.h"
#include "contador.h"
#include "lampara.h"
#include "sistema.h"
#include <iostream>

using namespace std;
using namespace unitpp;

namespace
{
	class PruebaEliminacion: public suite
	{
		#define DEBUG_ELIM
		/*	Funcion que crea un pulsador y nos los devuelve*/
		Pulsador* getPulsador()
		{
			Conexion* c1=new Conexion (CONEXION_INACTIVA);
			Conexion* c2=new Conexion (CONEXION_ACTIVA);
			Voltaje*  v1=new Voltaje  (0.0);
			Voltaje*  v2=new Voltaje  (5.0);
			#ifdef DEBUG_ELIM
				cout << "Creando pulsador\n";
			#endif
			Pulsador* p=new Pulsador(c1,v1,c2,v2);
			#ifdef DEBUG_ELIM
				cout << "Pulsador creado\n";
			#endif
			return p;
		}
		
		/*	Crea y nos devuelve un biestable*/
		Biestable* getBiestable()
		{
			Voltaje*  activar1	=	new Voltaje (5.0);
			Voltaje*  desactivar1=	new Voltaje (0.0);
			Voltaje*  activar2	=	new Voltaje (5.0);
			Voltaje*  desactivar2=	new Voltaje (0.0);
			Voltaje*  activar3	=	new Voltaje (5.0);
			Voltaje*  desactivar3=	new Voltaje (0.0);
			Voltaje*  activar4	=	new Voltaje (5.0);
			Voltaje*  desactivar4=	new Voltaje (0.0);
			#ifdef DEBUG_ELIM
				cout << "Creando biestable\n";
			#endif
			Biestable* b=new Biestable(activar1, desactivar1, 
				activar2,desactivar2, activar3, desactivar3,
				activar4,desactivar4);
			#ifdef DEBUG_ELIM
				cout << "Biestable creado\n";
			#endif
			return b;
		}
		Lampara* getLampara()
		{
			Voltaje*	v1=new Voltaje (0.0);
			Voltaje*	v2=new Voltaje (5.0);
			Luminosidad*l1=new Luminosidad("OFF");
			Luminosidad*l2=new Luminosidad("Rojo");
			#ifdef DEBUG_ELIM
				cout << "Creando lampara\n";
			#endif
			Lampara* lamp=new Lampara(v1,l1,v2,l2);
			#ifdef DEBUG_ELIM
				cout << "Lampara creada";
			#endif
			return lamp;
		}
		
	
		void pruebaEliminacionCompleja()
		{
			Voltaje *v1=new Voltaje(0.0);
			Voltaje *v2=new Voltaje(5.0);
			int numPulsos=3;
			Contador *c=new Contador(numPulsos, v1,v2,v1,v2);
			c->setNombreElemento("Contador 1");
			Sistema* subsubsistema=new Sistema();
			
			subsubsistema->anadirElemento(c);
			
			Elemento* recuperado=
				subsubsistema->getRefElementoPorNombre("Contador 1");
			assert_eq ("Contador dentro de subsubsistema", c, recuperado);
			
			Voltaje*  activar1	=	new Voltaje (5.0);
			Voltaje*  desactivar1=	new Voltaje (0.0);
			Voltaje*  activar2	=	new Voltaje (5.0);
			Voltaje*  desactivar2=	new Voltaje (0.0);
			Voltaje*  activar3	=	new Voltaje (5.0);
			Voltaje*  desactivar3=	new Voltaje (0.0);
			Voltaje*  activar4	=	new Voltaje (5.0);
			Voltaje*  desactivar4=	new Voltaje (0.0);
			
			Biestable* b=new Biestable(activar1, desactivar1, 
				activar2,desactivar2, activar3, desactivar3,
				activar4,desactivar4);
			b->setNombreElemento("Biestable 1");
			Sistema* subsistema=new Sistema();
			subsistema->anadirElemento(subsubsistema);
			subsistema->anadirElemento(b);
			Elemento* recuperado2=
				subsistema->getRefElementoPorNombre("Biestable 1");
			assert_eq ("Biestable dentro de subsistema", b, recuperado2);
			
			Conexion* c1=new Conexion (CONEXION_INACTIVA);
			Conexion* c2=new Conexion (CONEXION_ACTIVA);
			
			Pulsador* p=new Pulsador(c1,v1,c2,v2);
			Sistema* sistema=new Sistema();
			p->setNombreElemento("Pulsador 1");
			sistema->anadirElemento(p);
			sistema->anadirElemento(subsistema);
			Elemento* recuperado3=
				sistema->getRefElementoPorNombre("Pulsador 1");
			assert_eq ("Pulsador dentro de sistema", p, recuperado3);
			
			/*	Tenemos un subsubsistema con un contador dentro.
				Ese subsubsistema est dentro de un subsistema
				junto con un biestable.
				Ese subsistema est dentro de un sistema junto
				con un pulsador. Todo como si fuera una mueca rusa.
				Ahora se conectan el biestable al contador */
			int entradacontador=subsubsistema->getPrimeraEntradaUtil
				("Contador 1");
			/* Si es menor que 0 es que no se ha encontrador */
			bool encontrado = !(entradacontador<0);
			assert_eq ("Contador 1 encontrado en subsubsistema",
				true, encontrado);
			b->anadirSuscriptor(subsubsistema, entradacontador);
			
			int entradabiestable = subsistema->getPrimeraEntradaUtil
				("Biestable 1");
			encontrado = !(entradabiestable<0);
			assert_eq ("Biestable 1 encontrado en subsistema",
				true, encontrado);
			/*	La entradabiestable+1 es la entrada S*/
			p->anadirSuscriptor(subsistema, entradabiestable+1);
			
			/*	Activamos el pulsador */
			p->setEntrada(new Conexion(true));
			
			/*	Y se comprueba la salida */
			Voltaje* salidaBiestable= (Voltaje*)b->getSalidaPorPosicion();
			float valor= salidaBiestable->getValor();
			assert_eq ("Salida biestable", 5.0, valor);
			
			/*	Aqui viene toda la chicha de este test.
				Se borra el biestable y se vuelve a pulsar
				el pulsador, para ver qu pasa */
			subsistema->eliminarElemento(b);
			
			/*	Reinicio el estado del biestable*/
			b->setEntrada(new Voltaje(5.0));
			
			/*	Se pulsa el pulsador otra vez */
			p->setEntrada(new Conexion(false));
			
			/*	Esto debe fallar*/
			salidaBiestable= (Voltaje*)b->getSalidaPorPosicion();
			printf ("Recuperada la salida del biestable...");
			valor= salidaBiestable->getValor();
			printf ("...hecho\n");
			assert_eq ("Salida biestable", 5.0, valor);
		}
		public:
			PruebaEliminacion():suite ("Prueba Eliminacion")
			{
				add ("Prueba eliminacion compleja", testcase (this,
					"Prueba eliminacion compleja",
					 &PruebaEliminacion::pruebaEliminacionCompleja));
				suite::main().add("Prueba eliminacion", this);
			}
		
	};
	
	PruebaEliminacion* eliminar=new PruebaEliminacion();
}
